#ifndef AP_JAQPAR_H 
#define AP_JAQPAR_H 

#include <mpi.h>


/*-------------------------------------------------------------
C -Resolution du systeme A.X=B (de dimension NxN ) par 
C  la methode iterative de Jacobi par blocs. 
C -Soit A une matrice carree d'ordre N 
C      tridiagonale par blocs composee
C      de Nblocs blocs diagonaux de taille M (N = Nblocs*M)
C
C -EPS: definit la precision sur la solution recherchee.
C
C -Algorithme utilise: Version parallele de Jacobi par bloc
C Il sera suppose que le nombre de blocs Nblocs sera egal
C au nombre de processus MPI du communicateur

  Seul le processus maitre (processus 0) possede
  initialement la matrice tri-diagonale A et le second membre
  b. Le maitre distribue les donnees et 
  assemble la solution finale x}. Une fois cette distribution
  effectuee, il participe au calcul mais ne peut plsus acceder
  a A ni B.

  L'ordre N de la matrice A verifire N = Nblocs * M  ou
  Nprocs designe  le nombre 
  de processus MPI et est egal au nombre de blocs diagonaux.
  Le maitre distribue A et b afin que l'esclave k possede
  le k_ieme bloc ligne de A et b.

  Au cours des iterations chaque processus esclave k ne 
  connait que (et ne communique qu'avec) 
  ses processus voisins. (Avec la convention que le 
  voisin de droite de k a pour numero k+1 (ou 1 si k=Nblocs)
  et que le voisin de gauche a pour numero k-1 (ou Nprocs 
  si k=1).)

  Le processus k est en charge du calcul de x.
  A chaque etape de l'algorithme iteratif il met a jour x.
  Apres un echange entre processus voisins des 
  morceaux de solutions mises a jour, 
  il calcule un residu local r_k (k_ieme
  bloc ligne de r) et sa norme.
C -------------------------------------------------------------*/
/*----------
C Parametres: 
C ----------
C    Input  : A, B, N, M, Nblocs, EPS, Maxiter, precedant, suivant
              comm
C    Output : X, Niter
C
C    NB : ces parametres sont controles par l'utilisateur;
C    il est donc responsable de leur allocation et leur liberation.
C ----------------------------------*/
int jacpar(double *A, double *X, double *B, int N, int M, int Nblocs, double EPS, int Maxiter, int *Niter, int precedent, int suivant, MPI_Comm comm);

int jacpar_distentries(double *A, double *X, double *B, int N, int M, int Nblocs, double EPS, int Maxiter, int *Niter, int precedent, int suivant, MPI_Comm comm);


#endif
