#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <mpi.h>
#include "util.h"
#include "jacseq.h"
#include "jacpar.h"
  
int main(int argc, char **argv){
  double *A, *B, *X;
  double EPS, Max;
  int M, Nblocs, N, Maxiter, Niter, ierr;
  int argCount;
  int MaxMemoryAllocated, SumMemoryAllocated;
  int my_id, root, nb_proc;
  
  long int time;

  MPI_Comm comm = MPI_COMM_WORLD;
  MPI_Init(&argc,&argv);
  root = 0;
  MPI_Comm_rank(comm, &my_id);
  MPI_Comm_size(comm,&nb_proc);


  /* Initialisation du generateur de nombre pseudo-aleatoires */
  randomInit();

  M = 100;
  /*Nblocs = 10;*/
  Maxiter = 10;
  Max = 1.0;
  EPS = Max / 10000.0;
  
  for (argc--, argv++; argc > 0; argc -= argCount, argv += argCount){
    argCount = 1;
    if (!strcmp (*argv, "-h"))
      printf ("Usage: testjacob <-M bandsize> <-b Nblocs> <-i Maxiter> <-p precision>\n");
    else if (!strcmp (*argv, "-M")){
      ASSERT (argc > 1);
      M = atoi(*(argv + 1));
      printf("M redefini : M = %d\n",M);
      argCount = 2;
    }
    else if (!strcmp (*argv, "-b")){
      ASSERT (argc > 1);
      Nblocs = atoi(*(argv + 1));
      printf("Nblocs redefini : Nblocs = %d\n",Nblocs);
      argCount = 2;
    }
    else if (!strcmp (*argv, "-i")){
      ASSERT (argc > 1);
      Maxiter = atoi(*(argv + 1));
      printf("Maxiter redefini : Maxiter = %d\n",Maxiter);
      argCount = 2;
    }
    else if (!strcmp (*argv, "-E")){
      ASSERT (argc > 1);
      EPS = Max / atoi(*(argv + 1));
      printf("EPS redefini : EPS = %f\n", EPS);
      argCount = 2;
    }
    else{
      printf("testjacob: Unknown argument "); printf(*argv); printf("\n");
      printf ("Usage: testjacob <-M bandsize> <-b Nblocs> <-i Maxiter> <-p precision>\n");
      exit(0);
    }
  }
  

  Nblocs = nb_proc;
  N = Nblocs * M;

  if (my_id == root){
    A = malloc(M * M * (3 * Nblocs - 2) * sizeof(double));
    ASSERT (A != NULL);
    
    X = malloc(N * sizeof(double));
    ASSERT (X != NULL);
  
    B = malloc(N * sizeof(double));
    ASSERT (B != NULL);

    /* Generation de A et B */
    printf("Generation aleatoire de A et B avec Nblocs=%d, M=%d et Max=%e\n",Nblocs, M, Max);
    generateAB(A, B, M, Nblocs, Max);
    printf("A est d'ordre %d, contient %d elements non nuls (%d octets)\n", N, (M * M *(3 * Nblocs - 2)), (M * M *(3 * Nblocs - 2)) * sizeof(double));
    printf("A = ");
    printbuffer(A, M * M * (3 * Nblocs - 2), 0);
    printf("B = ");
    printbuffer(B, N, 0);
  }

  if (my_id == root){
    APMemoryAllocated = 0;
    time = ms_time();
    ierr = jacseq(A, X, B, N, M, Nblocs, EPS, Maxiter, &Niter);
    time = ms_time() - time;
    printf("Temps ecoule = %ld millisecondes\n", time);
    printf("Memoire allouee par jacseq = %d octets\n", APMemoryAllocated);
    printf("Nombre d'iterations = %d\n", Niter);
    printbuffer(X, N, 0);
  }

  APMemoryAllocated = 0;  
  time = ms_time();
  /*ierr = jacpar(A, X, B, N, M, Nblocs, EPS, Maxiter, &Niter, 0, 0, MPI_COMM_WORLD);*/
  time = ms_time() - time;
  fflush(stdout);
  
  MPI_Reduce (&APMemoryAllocated, &MaxMemoryAllocated, 1,
	      MPI_INT, MPI_MAX, root, comm);
  MPI_Reduce (&APMemoryAllocated, &SumMemoryAllocated, 1,
	      MPI_INT, MPI_SUM, root, comm);
  if (my_id == root){
    printf("Temps ecoule = %ld millisecondes\n", time);
    printf("Memoire max. par processeur allouee par jacpar = %d octets\n", MaxMemoryAllocated);
    printf("Memoire moyenne allouee par jacpar = %d octets\n", SumMemoryAllocated / nb_proc);
    printf("Nombre d'iterations = %d\n", Niter);
    printbuffer(X, N, 0);
  }
  
  if (my_id == root){
    free(A);
    free(X);
    free(B);
  }

  MPI_Finalize();
  return 0;
}
